SystemOrganization addCategory: #'Glamour-Morphic'!

GLMRenderer subclass: #GLMMorphicRenderer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

!GLMMorphicRenderer commentStamp: 'tg 5/24/2009 21:22' prior: 0!
This is the class responsible for the binding to Morphic.

| browser |
browser := GLMTableLayoutBrowser new.
browser column: #one; column: #two.

browser showOn: #one; using: [browser list.].
browser showOn: #two; from: #one; using: [
	browser text.].
browser openOn: #(a b c d)!

!GLMMorphicRenderer methodsFor: 'factory' stamp: ''!
emptyMorph
	^ RectangleMorph new
		layoutFrame: (LayoutFrame fractions: (0 @ 0 corner: 1 @ 1));
		layoutPolicy: ProportionalLayout new;
		color: Color transparent;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 0;
		yourself! !

!GLMMorphicRenderer methodsFor: 'private' stamp: ''!
installActionsOnUI: aMorphicModel fromPresentation: aPresentation	 
	aPresentation actions do: [ :action |  
		"menu actions"
		aMorphicModel when: GLMMenuItemSelected do: [ :ann | ann action actOn: aPresentation ].
		action hasTitle ifTrue: [ aMorphicModel addMenuAction: action ].
			
		"keystroke actions"
		aMorphicModel when: GLMKeyStroke do: [ :ann | ann action actOn: aPresentation ].			
		action hasShortcut ifTrue: [ aMorphicModel addKeystrokeAction: action ] ]! !

!GLMMorphicRenderer methodsFor: 'opening' stamp: ''!
open: aRenderable
	| window |
	window := GlamorousSystemWindow new.
	window model: self; setLabel: aRenderable titleValue.
	window addMorph: (self render: aRenderable) frame: (0 @ 0 corner: 1 @ 1).
	^ window openInWorld! !

!GLMMorphicRenderer methodsFor: 'private' stamp: ''!
renderCustomColumn: aCell ofPane: aPane inUI: aUIContainer inBrowser: aBrowser 
	| pane totalSpans currentSpanPosition |
	totalSpans := aCell children
		inject: 0
		into: [ :sum :each | sum + each span ].
	currentSpanPosition := 0.
	aCell children keysAndValuesDo: [ :index :each | 
		each hasId
			ifTrue: [ pane := self render: (aBrowser paneNamed: each id) ]
			ifFalse: [
				self
					renderCustomRow: each
					ofPane: aPane
					inUI: (pane := self emptyMorph)
					inBrowser: aBrowser.
				pane addPaneSplitters ].
		pane 
			layoutFrame: (LayoutFrame new
				topFraction: currentSpanPosition / totalSpans offset: 0;
				leftFraction: 0 offset: 0;
				bottomFraction: currentSpanPosition + each span / totalSpans offset: 0;
				rightFraction: 1 offset: 0;
				yourself).
		aUIContainer addMorphBack: pane.
		currentSpanPosition := currentSpanPosition + each span ]! !

!GLMMorphicRenderer methodsFor: 'private' stamp: ''!
renderCustomRow: aCell ofPane: aPane inUI: aUIContainer inBrowser: aBrowser 
	| pane totalSpans currentSpanPosition |
	totalSpans := aCell children
		inject: 0
		into: [ :sum :each | sum + each span ].
	currentSpanPosition := 0.
	aCell children keysAndValuesDo: [ :index :each | 
		each hasId
			ifTrue: [ pane := self render: (aBrowser paneNamed: each id) ]
			ifFalse: [
				self
					renderCustomColumn: each
					ofPane: aPane
					inUI: (pane := self emptyMorph)
					inBrowser: aBrowser.
				pane addPaneSplitters ].
		pane
			layoutFrame: (LayoutFrame new 
				topFraction: 0 offset: 0;
				leftFraction: currentSpanPosition / totalSpans offset: 0;
				bottomFraction: 1 offset: 0;
				rightFraction: currentSpanPosition + each span / totalSpans offset: 0;
				yourself).
		aUIContainer addMorphBack: pane.
		currentSpanPosition := currentSpanPosition + each span ]! !

!GLMMorphicRenderer methodsFor: 'rendering browsers' stamp: 'jorge.ressia 5/28/2009 09:28'!
renderFinder: aBrowser 
	| container |
	container := GLMPaneScroller new
		name: 'scroller';
		vResizing: #spaceFill;
		hResizing: #spaceFill;
		yourself.
	aBrowser 
		when: GLMPaneRemoved
		do: [ :ann | container popPane.].
	aBrowser 
		when: GLMPaneAdded
		do: [ :ann | container pushPane: (ann pane renderGlamorouslyOn: self).].
	aBrowser panes do: [ :each | container pushPane: (each renderGlamorouslyOn: self) ].
	^ container! !

!GLMMorphicRenderer methodsFor: 'rendering presentations' stamp: ''!
renderListPresentation: aListPresentation 
	| items list toShow listModel container search |
	toShow := aListPresentation displayValue.
	items := toShow isCollection
				ifTrue: [toShow]
				ifFalse: [Array with: toShow].
	listModel := GLMListModel new
				items: (items
						ifNil: [OrderedCollection new]);
				 displaySelector: aListPresentation format;
				 searchBlock: aListPresentation searchStrategy;
				 filterBlock: aListPresentation filterStrategy;
				 yourself.
	container := self emptyMorph.
	list := aListPresentation isMultiple
				ifTrue: [
					PluggableListMorphOfMany
						on: listModel
						list: #items
						primarySelection: #selection
						changePrimarySelection: #selection:
						listSelection: #listSelectionAt:
						changeListSelection: #listSelectionAt:put:
						menu: #menu
						keystroke: #keystroke:from:]
				ifFalse: [
					self theme 
						newListIn: container 
						for: listModel 
						list: #items 
						selected: #singleSelection 
						changeSelected: #singleSelection:
						getEnabled: nil 
						help: nil ].
	list
		getMenuSelector: #menu:;
		keystrokeActionSelector: #keystroke:from:;
		layoutFrame: (LayoutFrame fractions: (0 @ 0 corner: 1 @ 1)).
	self installActionsOnUI: listModel fromPresentation: aListPresentation.
	listModel
		onSelection: [:selection | aListPresentation selection: selection].
	aListPresentation isSearchable ifTrue: [
		search := UITheme current
			newTextEntryIn: nil
			for: listModel
			get: #searchText
			set: #searchText:
			class: String
			getEnabled: #searchEnabled
			help: 'Search Input'.
		search layoutFrame: (LayoutFrame
			fractions: (0 @ 1 corner: 1 @ 1) 
			offsets: (0 @ -24 corner: 0 @ 0)).
		list layoutFrame bottomOffset: -24.
		container addMorphBack: search ].
	container addMorphBack: list.
	^ container! !

!GLMMorphicRenderer methodsFor: 'rendering presentations' stamp: 'tg 5/24/2009 21:50'!
renderMagrittePresentation: anAsMorphPresentation 
	| toShow description magritteMorph scrollPane |
	
	toShow := anAsMorphPresentation displayValue.
	description := (anAsMorphPresentation magritteDescription glamourValue: toShow) copy.
	description morphClass: MASilentContainerMorph.
	magritteMorph := (description asMorphOn: toShow) 
		addButtons;
		morph.
	scrollPane := ScrollPane new.
	scrollPane scroller addMorph: magritteMorph.	
	^ scrollPane
! !

!GLMMorphicRenderer methodsFor: 'rendering presentations' stamp: 'tg 5/20/2009 16:53'!
renderMondrianPresentation: aMondrianPresentation 
	 
	| canvas view canvasScrollPane | 
	view := MOViewRenderer new.
	aMondrianPresentation renderOn: view. 
	canvas := MOCanvas new. 
	canvas root: view root.
	canvas announcer
		when: MOFigureSelection
		do: [ :ann | aMondrianPresentation selection: ann element model ].
	canvasScrollPane := ScrollPane new.
	canvasScrollPane scroller addMorph: canvas.	
	canvasScrollPane
			vResizing: #spaceFill;
			hResizing: #spaceFill.
	^ canvasScrollPane
	

"	aMondrianPresentation 
		whenContextChanged: 
			[:ann |  
			ann property = #selection 
				ifTrue: 
					[canvas artist drawing deepNodeFigures do: [:each | each deselect]. 
					(canvas artist drawing deepFindNodeWithEntity: ann value) 
						ifNotNil: [:figure | figure fixedSelect]]]. 

	scrollPane := GeneralScrollPane new
		scrollTarget: canvas;
		hResizing: #spaceFill;
		vResizing: #spaceFill.
	scrollPane
		minWidth: (canvas width min: Display width // 2) + scrollPane scrollBarThickness;
		minHeight: (canvas height min: Display height // 2).
	canvas width > scrollPane minWidth
		ifTrue: [scrollPane minHeight: scrollPane minHeight + scrollPane scrollBarThickness].

	scrollPane updateScrollbars.
	^ scrollPane
"! !

!GLMMorphicRenderer methodsFor: 'rendering' stamp: ''!
renderPane: aPane 
	 
	| container inspectButton label | 
	container := self emptyMorph.

	(aPane matchingPresentations anySatisfy: [:each | each isInspectable]) ifTrue: [
		label := PluggableTextMorph new. 
		label layoutInset: 2. 
		label setText: (aPane port: #entity) value displayString. 
		inspectButton := PluggableButtonMorph new. 
		inspectButton label: (aPane port: #entity) value displayString.
		container addMorphBack: inspectButton ].
	
	container addMorphBack: (self renderPresentationsOf: aPane).
	aPane whenMatchingPresentationsChanged: [:ann |  
		container removeAllMorphs.
		container addMorph: (self renderPresentationsOf: aPane) ].
	container addPaneSplitters.
	^ container! !

!GLMMorphicRenderer methodsFor: 'private' stamp: ''!
renderPresentationsOf: aPane
	| presentations tabs | 
	presentations := aPane matchingPresentations.
	presentations isEmpty
		ifTrue: [ ^ self emptyMorph ].
	presentations size = 1
		ifTrue: [ ^ self render: presentations first ].
	tabs := TabGroupMorph new. 
	tabs vResizing: #spaceFill; hResizing: #spaceFill.
	presentations do: [ :each | 
		tabs
			addPage: (self render: each)
			label: (each titleValue ifNil: [ each printString ]) ].
	tabs selectedPageIndex: 1.
	^ tabs! !

!GLMMorphicRenderer methodsFor: 'rendering browsers' stamp: ''!
renderTableLayoutBrowser: aBrowser 
	| container |
	container := self emptyMorph.
	aBrowser cell isRow
		ifTrue: [
			self
				renderCustomRow: aBrowser cell
				ofPane: aBrowser pane
				inUI: container
				inBrowser: aBrowser ]
		ifFalse: [
			self
				renderCustomColumn: aBrowser cell
				ofPane: aBrowser pane
				inUI: container
				inBrowser: aBrowser ].
	^ container! !

!GLMMorphicRenderer methodsFor: 'rendering presentations' stamp: ''!
renderTextPresentation: aTextPresentation 
	| textMorph toShow textModel | 
	toShow := aTextPresentation formatedDisplayValue. 
	textModel := GLMTextModel new text: toShow.
	textModel onTextChanged: [:text | aTextPresentation text: text].
	textMorph := (PluggableTextMorph 
		on: textModel
		text: #text
		accept: #accept:notifying:
		readSelection: #selection
		menu: #menu:shifted:)
			onKeyStrokeSend: #keystroke:from: to: textModel;
			layoutFrame: (LayoutFrame fractions: (0 @ 0 corner: 1 @ 1));
			vResizing: #spaceFill;
			hResizing: #spaceFill;
			yourself.
	self installActionsOnUI: textModel fromPresentation: aTextPresentation.
	^textMorph! !

!GLMMorphicRenderer methodsFor: 'rendering presentations' stamp: ''!
renderTreePresentation: aTreePresentation	 
	| toShow treeMorph treeModel | 
	
	toShow := aTreePresentation displayValue. 
	treeModel := GLMTreeModel new roots: toShow; yourself.
	treeModel 
			childrenBlock: aTreePresentation children;
			displaySelector: aTreePresentation format.
	
	treeMorph := PluggableTreeMorph new
		model: treeModel;
		getLabelSelector: #getLabelOf:;		
		getSelectedPathSelector: #getSelectionPath;				
		getChildrenSelector: #getChildrenOf:;
		hasChildrenSelector: #hasChildren:;
		setSelectedSelector: #selection:;					
		getRootsSelector: #roots;
		keystrokeActionSelector: #keystroke:from:;		
		layoutFrame: (LayoutFrame fractions: (0 @ 0 corner: 1 @ 1));
		hResizing: #spaceFill; vResizing: #spaceFill;
		autoDeselect: false;
		yourself.
	self installActionsOnUI: treeModel fromPresentation: aTreePresentation.
	
	treeModel onSelection: [:selection | aTreePresentation selection: selection]. 
	treeMorph vShowScrollBar.	
	^treeMorph! !

!GLMMorphicRenderer methodsFor: 'rendering presentations' stamp: 'tg 5/24/2009 19:29'!
renderWorkspacePresentation: aWorkspacePresentation 
	 
	^ self renderTextPresentation: aWorkspacePresentation! !

!GLMMorphicRenderer methodsFor: 'factory' stamp: ''!
theme
	^ UITheme current! !

Announcement subclass: #GLMActionAnnouncement
	instanceVariableNames: 'action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

!GLMActionAnnouncement classSide methodsFor: 'instance creation' stamp: 'david_roethlisberger 3/3/2009 16:28'!
action: anAction
	^self new action: anAction; yourself! !

!GLMActionAnnouncement methodsFor: 'accessing' stamp: 'david_roethlisberger 3/3/2009 16:28'!
action
	^action! !

!GLMActionAnnouncement methodsFor: 'accessing' stamp: 'david_roethlisberger 3/3/2009 16:28'!
action: anAction
	action := anAction! !

GLMActionAnnouncement subclass: #GLMKeyStroke
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

GLMActionAnnouncement subclass: #GLMMenuItemSelected
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

RectangleMorph subclass: #GLMPaneScroller
	instanceVariableNames: 'sizing transform scrollBar paneCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

!GLMPaneScroller methodsFor: 'layout' stamp: 'david_roethlisberger 3/18/2009 14:43'!
adjustPaneHeight
	"This gets called after the scrollbar has been shown or hidden, to move the bottom
	of the panes to the right place."
	
	transform bounds: self innerBounds.
	transform submorphsDo: [:m | m bounds: (m bounds withHeight: self paneHeight)]! !

!GLMPaneScroller methodsFor: 'private' stamp: 'jorge.ressia 5/25/2009 16:25'!
basicUpdateSizing
	sizing := (self paneCount min: self maxPanes) max: 1! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'david_roethlisberger 3/18/2009 14:21'!
bounds: aRectangle
	super bounds: aRectangle.
	self layoutWidgets.
	self layoutPanes.
	self setScrollDeltas.
! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'david_roethlisberger 3/18/2009 14:21'!
computeMorphWidths
	| paneWidths widths |
	paneWidths := self paneWidthsToFit: self totalPaneWidth.
	widths := OrderedCollection new.
	paneWidths do: [:w | widths add: w] separatedBy: [widths add: self separatorWidth].
	^ widths asArray
! !

!GLMPaneScroller methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/18/2009 14:25'!
hideOrShowScrollBar
	self isScrollable ifTrue: [self showScrollBar] ifFalse: [self hideScrollBar]! !

!GLMPaneScroller methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/18/2009 14:25'!
hideScrollBar
	self removeMorph: scrollBar.
	self adjustPaneHeight.! !

!GLMPaneScroller methodsFor: 'initialization' stamp: 'jorge.ressia 5/25/2009 16:29'!
initialize
	super initialize.
	self 
		color: Color white;
		borderWidth: 0;
		vResizing: #spaceFill;
		hResizing: #spaceFill.
	self 
		initializeTransform;
		initializeScrollbar.
	paneCount := 0.! !

!GLMPaneScroller methodsFor: 'initialization' stamp: 'david_roethlisberger 3/19/2009 10:41'!
initializeScrollbar
	scrollBar := GLMScrollBar new model: self slotName: 'scrollBar'.
	scrollBar 
		borderWidth: 0;
		borderColor: #inset;
		height: self scrollBarHeight.
	self resizeScrollBar.
! !

!GLMPaneScroller methodsFor: 'initialization' stamp: 'david_roethlisberger 3/18/2009 14:26'!
initializeTransform
	transform := TransformMorph new.
	transform
		color: Color transparent;
		borderWidth: 0;
		vResizing: #spaceFill;
		hResizing: #spaceFill;
		disableTableLayout;
		bounds: super innerBounds.
	self addMorphBack: transform.
! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'david_roethlisberger 3/18/2009 14:21'!
innerBounds
	| rect |
	rect := super innerBounds.
	^ self scrollBarIsVisible
		ifTrue: [rect withHeight: rect height - self scrollBarHeight - 1]
		ifFalse: [rect]! !

!GLMPaneScroller methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/18/2009 14:25'!
isScrollable
	^ self leftoverScrollRange > 0! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'david_roethlisberger 3/18/2009 14:21'!
layoutPanes
	| widths rect |
	widths := self computeMorphWidths.
	rect := 0@0 extent: (0 @ self paneHeight).
	transform submorphs 
					with: widths 
					do: [:m :w | 
						rect := rect withWidth: w.
						m bounds: rect.
						rect := rect translateBy: (w@0)]
						
! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'david_roethlisberger 3/18/2009 14:21'!
layoutWidgets
	| inner outer |
	outer := super innerBounds.
	inner := self innerBounds.
	transform bounds: inner.
	scrollBar bounds: ((inner left @ inner bottom + 1)
						corner: outer bottomRight)! !

!GLMPaneScroller methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/18/2009 14:25'!
leftoverScrollRange
	^ (self totalScrollRange - self innerBounds width roundTo: self scrollDeltaWidth) max: 0
! !

!GLMPaneScroller methodsFor: 'configuration' stamp: 'david_roethlisberger 3/18/2009 15:05'!
maxPanes
	^2! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'jorge.ressia 5/25/2009 16:18'!
paneCount
	^ paneCount! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'david_roethlisberger 3/18/2009 14:22'!
paneHeight
	^ transform bounds height! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'david_roethlisberger 3/18/2009 14:22'!
paneWidthsToFit: limit
	| padded |
	padded := Array new: self paneCount.
	padded atAllPut: (limit / self sizing) floor.
	(1 to: limit - padded sum) do: [:i | padded at: i put: (padded at: i) + 1].
	^ padded
	
	! !

!GLMPaneScroller methodsFor: 'panes' stamp: 'jorge.ressia 5/25/2009 16:41'!
popPane
	transform removeMorph: transform lastSubmorph. "one for the separator"
	transform removeMorph: transform lastSubmorph.
	paneCount := paneCount - 1.
	self updatePanes.! !

!GLMPaneScroller methodsFor: 'panes' stamp: 'jorge.ressia 5/25/2009 16:41'!
pushPane: aMorph	
	aMorph 
		borderWidth: 0;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0.
	transform hasSubmorphs ifTrue: [transform addMorphBack: self separator].
	transform addMorphBack: aMorph.
	paneCount := paneCount + 1.
	self updatePanes! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'jorge.ressia 5/25/2009 16:29'!
resizeScrollBar
	| inner outer |
	outer := super innerBounds.
	inner := outer withHeight: outer height - self scrollBarHeight - 1.
	scrollBar bounds: ((inner left @ inner bottom + 1) corner: outer bottomRight)! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'david_roethlisberger 3/18/2009 14:22'!
scrollBarHeight
	^ 12! !

!GLMPaneScroller methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/18/2009 14:25'!
scrollBarIsVisible
	^ submorphs includes: scrollBar! !

!GLMPaneScroller methodsFor: 'updating' stamp: 'david_roethlisberger 3/18/2009 14:27'!
scrollBarValue: value
	transform hasSubmorphs ifFalse: [^ self].
	transform offset: (self leftoverScrollRange * value) rounded @ 0.! !

!GLMPaneScroller methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/18/2009 14:25'!
scrollDeltaWidth
	^ 1! !

!GLMPaneScroller methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/18/2009 14:25'!
scrollToRight
	^ scrollBar setValue: 1.! !

!GLMPaneScroller methodsFor: 'panes' stamp: 'jorge.ressia 5/25/2009 16:43'!
separator
	^ BorderedSubpaneDividerMorph vertical
		color: self defaultBackgroundColor duller;
		yourself! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'david_roethlisberger 3/18/2009 14:28'!
separatorWidth
	^3! !

!GLMPaneScroller methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/18/2009 14:26'!
setScrollDeltas
	| range interval value |
	transform hasSubmorphs ifFalse: [scrollBar interval: 1.0. ^ self].
	range := self leftoverScrollRange.
	range = 0 ifTrue: [^ scrollBar interval: 1.0; setValue: 0].
	interval := ((self innerBounds width) / self totalScrollRange) asFloat.
	value := (transform offset x / range min: 1.0) asFloat.
	scrollBar interval: interval.
	scrollBar setValue: value.! !

!GLMPaneScroller methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/18/2009 14:26'!
showScrollBar
	self scrollBarIsVisible ifTrue: [^ self].
	self resizeScrollBar.
	self addMorphFront: scrollBar.
	self adjustPaneHeight.
	! !

!GLMPaneScroller methodsFor: 'accessing' stamp: 'david_roethlisberger 3/18/2009 14:27'!
sizing
	^ sizing ifNil: [self updateSizing]! !

!GLMPaneScroller methodsFor: 'layout' stamp: 'david_roethlisberger 3/18/2009 14:22'!
totalPaneWidth
	^ self innerBounds width - ((self sizing - 1) * self separatorWidth)! !

!GLMPaneScroller methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/18/2009 14:26'!
totalScrollRange
	| submorphBounds |
	submorphBounds := transform localSubmorphBounds ifNil: [^ 0].
	^ submorphBounds width
! !

!GLMPaneScroller methodsFor: 'updating' stamp: 'jorge.ressia 5/25/2009 16:40'!
updatePanes
	self basicUpdateSizing.
	self layoutPanes.
	self hideOrShowScrollBar.
	self setScrollDeltas.
	self scrollToRight! !

!GLMPaneScroller methodsFor: 'updating' stamp: 'david_roethlisberger 3/18/2009 14:27'!
updateSizing
	| old |
	old := sizing.
	self basicUpdateSizing.
	sizing = old ifFalse: [self layoutPanes].
	^sizing! !

SystemWindow subclass: #GlamorousSystemWindow
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

!GlamorousSystemWindow methodsFor: 'building' stamp: ''!
buildWindowMenu
	^ self model actions isEmpty
		ifTrue: [ super buildWindowMenu ]
		ifFalse: [
			self model actions
				inject: (super buildWindowMenu
					addLine;
					yourself)
				into: [ :menu :action |
					menu
						add: action title target: action selector: #actOn: argument: self model;
						yourself ] ]! !

Slider subclass: #GLMScrollBar
	instanceVariableNames: 'upButton downButton pagingArea scrollDelta pageDelta interval menuSelector timeOfMouseDown timeOfLastScroll nextPageDirection currentScrollDelay'
	classVariableNames: 'CachedImages UpArrow UpArrow8Bit'
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

!GLMScrollBar classSide methodsFor: 'as yet unclassified' stamp: 'david_roethlisberger 3/19/2009 10:40'!
alwaysShowFlatScrollbarForAlternativeLook
	"Set this value to true, if you want to see the flat scrollbar look in flop-out mode as well as inboard.  Otherwise the flop-out scrollbar will be rounded and inboard will be flat."
	^ false! !

!GLMScrollBar classSide methodsFor: 'class initialization' stamp: 'david_roethlisberger 4/1/2009 13:51'!
initialize "GLMScrollBar initialize"

	UpArrow := Form
		extent: 6@3
		fromArray: #(2r11e28 2r1111e27 2r111111e26)
		offset: 0@0.! !

!GLMScrollBar methodsFor: 'accessing' stamp: 'david_roethlisberger 3/19/2009 10:40'!
adoptPaneColor: aColor
	"Adopt the given pane color"
	super adoptPaneColor: aColor.
	aColor ifNil:[^self].
	self sliderColor: aColor.! !

!GLMScrollBar methodsFor: 'geometry' stamp: 'david_roethlisberger 3/19/2009 10:40'!
buttonExtent
	^ bounds isWide
		ifTrue: [11 @ self innerBounds height]
		ifFalse: [self innerBounds width @ 11]! !

!GLMScrollBar methodsFor: 'accessing' stamp: 'david_roethlisberger 3/19/2009 10:40'!
cachedImageAt: aKey ifAbsentPut: aBlock

	CachedImages ifNil: [CachedImages := Dictionary new].
	^CachedImages at: aKey ifAbsentPut: aBlock! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
doScrollByPage
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 300 delay2: 100) ifFalse: [^ self].
	nextPageDirection
		ifTrue: [self setValue: (value + pageDelta min: 1.0)]
		ifFalse: [self setValue: (value - pageDelta max: 0.0)]
! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
doScrollDown
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 200 delay2: 40) ifFalse: [^ self].
	self setValue: (value + scrollDelta + 0.000001 min: 1.0)! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
doScrollUp
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 200 delay2: 40) ifFalse: [^ self].
	self setValue: (value - scrollDelta - 0.000001 max: 0.0)! !

!GLMScrollBar methodsFor: 'geometry' stamp: 'david_roethlisberger 3/19/2009 10:40'!
expandSlider
	"Compute the new size of the slider (use the old sliderThickness as a minimum)."
	| r |
	r := self totalSliderArea.
	slider extent: (bounds isWide
		ifTrue: [((r width * interval) asInteger max: self sliderThickness) @ slider height]
		ifFalse: [slider width @ ((r height * interval) asInteger max: self sliderThickness)])! !

!GLMScrollBar methodsFor: 'geometry' stamp: 'david_roethlisberger 3/19/2009 10:40'!
extent: p
        p x > p y
                ifTrue: [super extent: (p max: 42@8)]
                ifFalse: [super extent: (p max: 8@42)]! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
finishedScrolling
	self stopStepping.
	self scrollBarAction: nil.
	self roundedScrollbarLook ifTrue:[
		upButton borderStyle: (BorderStyle complexRaised width: upButton borderWidth).
		downButton borderStyle: (BorderStyle complexRaised width: downButton borderWidth).
	] ifFalse:[
		downButton borderRaised.
		upButton borderRaised.
	].

! !

!GLMScrollBar methodsFor: 'initialization' stamp: 'david_roethlisberger 3/19/2009 10:40'!
initialize
	super initialize.
	scrollDelta := 0.02.
	pageDelta := 0.2.
	self roundedScrollbarLook ifTrue:[
		self borderStyle: ((BorderStyle complexFramed width: 2) "baseColor: Color gray")].! !

!GLMScrollBar methodsFor: 'initialize' stamp: 'david_roethlisberger 3/19/2009 10:40'!
initializeDownButton
	downButton := RectangleMorph
		newBounds: (self innerBounds bottomRight - self buttonExtent extent: self buttonExtent)
		color: self thumbColor.
	downButton on: #mouseDown send: #scrollDownInit to: self.
	downButton on: #mouseUp send: #finishedScrolling to: self.
	downButton addMorphCentered: (ImageMorph new image: 
		(self 
			cachedImageAt: (bounds isWide ifTrue: ['right'] ifFalse: ['down']) 
			ifAbsentPut: [
				self upArrow8Bit
					rotateBy: (bounds isWide ifTrue: [#right] ifFalse: [#pi])
					centerAt: 0@0
			]
		)
	).
	self roundedScrollbarLook ifTrue:[
		downButton color: Color veryLightGray.
		downButton borderStyle: (BorderStyle complexRaised width: 3).
	] ifFalse:[
		downButton setBorderWidth: 1 borderColor: #raised.
	].
	self addMorph: downButton.
! !

!GLMScrollBar methodsFor: 'initialize' stamp: 'david_roethlisberger 3/19/2009 10:40'!
initializeEmbedded: aBool
	"aBool == true => inboard scrollbar
	aBool == false => flop-out scrollbar"
	self roundedScrollbarLook ifFalse:[^self].
	aBool ifTrue:[
		self borderStyle: (BorderStyle inset width: 2).
		self cornerStyle: #square.
	] ifFalse:[
		self borderStyle: (BorderStyle width: 1 color: Color black).
		self cornerStyle: #rounded.
	].
	self removeAllMorphs.
	self initializeSlider.! !

!GLMScrollBar methodsFor: 'initialize' stamp: 'david_roethlisberger 3/19/2009 10:40'!
initializePagingArea
	pagingArea := RectangleMorph newBounds: self totalSliderArea
								color: (Color r: 0.6 g: 0.6 b: 0.8).
	pagingArea borderWidth: 0.
	pagingArea on: #mouseDown send: #scrollPageInit: to: self.
	pagingArea on: #mouseUp send: #finishedScrolling to: self.
	self addMorph: pagingArea.
	self roundedScrollbarLook 
		ifTrue:[pagingArea color: (Color gray: 0.9)].! !

!GLMScrollBar methodsFor: 'initialize' stamp: 'david_roethlisberger 3/19/2009 10:40'!
initializeSlider
	self roundedScrollbarLook ifTrue:[
		self 
			initializeUpButton;
			initializeDownButton;
			initializePagingArea.
	] ifFalse:[
		self initializeUpButton; 
			initializeDownButton; 
			initializePagingArea.
	].
	super initializeSlider.
	self roundedScrollbarLook ifTrue:[
		slider cornerStyle: #rounded.
		slider borderStyle: (BorderStyle complexRaised width: 3).
		sliderShadow cornerStyle: #rounded.
	].
	self sliderColor: self sliderColor.! !

!GLMScrollBar methodsFor: 'initialize' stamp: 'david_roethlisberger 3/19/2009 10:40'!
initializeUpButton
	upButton := self roundedScrollbarLook 
				ifTrue:  
					[RectangleMorph 
						newBounds: (self innerBounds topLeft extent: self buttonExtent)]
				ifFalse: 
					[RectangleMorph 
						newBounds: ((self innerBounds topLeft) extent: self buttonExtent)].
	upButton color: self thumbColor.
	upButton 
		on: #mouseDown
		send: #scrollUpInit
		to: self.
	upButton 
		on: #mouseUp
		send: #finishedScrolling
		to: self.
	upButton 
		addMorphCentered: (ImageMorph new image: (self 
						cachedImageAt: (bounds isWide ifTrue: ['left'] ifFalse: ['up'])
						ifAbsentPut: 
							[bounds isWide 
								ifTrue: [self upArrow8Bit rotateBy: #left centerAt: 0 @ 0]
								ifFalse: [self upArrow8Bit]])).
	self roundedScrollbarLook 
		ifTrue: 
			[upButton color: Color veryLightGray.
			upButton borderStyle: (BorderStyle complexRaised width: 3)]
		ifFalse: [upButton setBorderWidth: 1 borderColor: #raised].
	self addMorph: upButton! !

!GLMScrollBar methodsFor: 'access' stamp: 'david_roethlisberger 3/19/2009 10:40'!
interval: d
	"Supply an optional floating fraction so slider can expand to indicate range"
	interval := d min: 1.0.
	self expandSlider.
	self computeSlider.! !

!GLMScrollBar methodsFor: 'other events' stamp: 'david_roethlisberger 3/19/2009 10:40'!
menuButtonMouseDown: event
	event hand showTemporaryCursor: nil.
	self use: menuSelector orMakeModelSelectorFor: 'MenuButtonPressed:'
		in: [:sel | menuSelector := sel.  model perform: sel with: event]! !

!GLMScrollBar methodsFor: 'other events' stamp: 'david_roethlisberger 3/19/2009 10:40'!
mouseDownInSlider: event
	interval = 1.0 ifTrue:
		["make the entire scrollable area visible if a full scrollbar is clicked on"
		self setValue: 0.
		self model hideOrShowScrollBar].
	super mouseDownInSlider: event! !

!GLMScrollBar methodsFor: 'access' stamp: 'david_roethlisberger 3/19/2009 10:40'!
pagingArea
	^pagingArea! !

!GLMScrollBar methodsFor: 'scroll timing' stamp: 'david_roethlisberger 3/19/2009 10:40'!
resetTimer
	timeOfMouseDown := Time millisecondClockValue.
	timeOfLastScroll := timeOfMouseDown - 1000 max: 0.
	nextPageDirection := nil.
	currentScrollDelay := nil! !

!GLMScrollBar methodsFor: 'access' stamp: 'david_roethlisberger 3/19/2009 10:40'!
roundedScrollbarLook
	"Rounded look currently only shows up in flop-out mode"
	^false and: [self class alwaysShowFlatScrollbarForAlternativeLook not]
! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
scrollBarAction
	^self valueOfProperty: #scrollBarAction! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
scrollBarAction: aSymbol
	self setProperty: #scrollBarAction toValue: aSymbol! !

!GLMScrollBar methodsFor: 'access' stamp: 'david_roethlisberger 3/19/2009 10:40'!
scrollDelta
	^ scrollDelta! !

!GLMScrollBar methodsFor: 'access' stamp: 'david_roethlisberger 3/19/2009 10:40'!
scrollDelta: d1 pageDelta: d2
	"Supply optional increments for better scrolling of, eg, text"
	scrollDelta := d1.
	pageDelta := d2.! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
scrollDown
	self flag: #obsolete.
	downButton eventHandler: nil.
	downButton on: #mouseDown send: #scrollDownInit to: self.
	downButton on: #mouseUp send: #finishedScrolling to: self.
	^self scrollDownInit! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
scrollDown: count
	self setValue: (value + (scrollDelta * count) + 0.000001 min: 1.0)! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
scrollDownInit
	downButton borderInset.
	self resetTimer.
	self scrollBarAction: #doScrollDown.
	self startStepping.! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
scrollPageInit: evt
	self resetTimer.
	self setNextDirectionFromEvent: evt.
	self scrollBarAction: #doScrollByPage.
	self startStepping.! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
scrollUp
	self flag: #obsolete.
	upButton eventHandler: nil.
	upButton on: #mouseDown send: #scrollUpInit to: self.
	upButton on: #mouseUp send: #finishedScrolling to: self.
	^self scrollUpInit! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
scrollUp: count
	self setValue: (value - (scrollDelta * count) - 0.000001 max: 0.0)! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
scrollUpInit
	upButton borderInset.
	self resetTimer.
	self scrollBarAction: #doScrollUp.
	self startStepping.! !

!GLMScrollBar methodsFor: 'scrolling' stamp: 'david_roethlisberger 3/19/2009 10:40'!
setNextDirectionFromEvent: event

	nextPageDirection := bounds isWide ifTrue: [
		event cursorPoint x >= slider center x
	]
	ifFalse: [
		event cursorPoint y >= slider center y
	]

! !

!GLMScrollBar methodsFor: 'model access' stamp: 'david_roethlisberger 3/19/2009 10:40'!
setValue: newValue
	"Using roundTo: instead of truncateTo: ensures that scrollUp will scroll the same distance as scrollDown."
	^ super setValue: (newValue roundTo: scrollDelta)! !

!GLMScrollBar methodsFor: 'access' stamp: 'david_roethlisberger 3/19/2009 10:40'!
sliderColor: aColor
	"Change the color of the scrollbar to go with aColor."
	| buttonColor |
	super sliderColor: aColor.
	buttonColor := self thumbColor.
	upButton color: buttonColor.
	downButton color: buttonColor.
	slider color: buttonColor.
	self roundedScrollbarLook
			ifTrue:
				[self color: Color transparent.
				pagingArea color: aColor muchLighter.
				self borderStyle style == #simple 
					ifTrue:[self borderColor: aColor darker darker]
					ifFalse:[self borderStyle baseColor: aColor]]
			ifFalse:
				[pagingArea color: (aColor alphaMixed: 0.3 with: Color white).
				self borderWidth: 0]
! !

!GLMScrollBar methodsFor: 'geometry' stamp: 'david_roethlisberger 3/19/2009 10:40'!
sliderExtent
	"The sliderExtent is now stored in the slider itself, not hardcoded as it is in the superclass."
	^slider extent! !

!GLMScrollBar methodsFor: 'access' stamp: 'david_roethlisberger 3/19/2009 10:40'!
sliderShadowColor
	^ self roundedScrollbarLook
		ifTrue: [self sliderColor darker]
		ifFalse: [super sliderShadowColor]
! !

!GLMScrollBar methodsFor: 'geometry' stamp: 'david_roethlisberger 3/19/2009 10:40'!
sliderThickness
	^ self roundedScrollbarLook ifTrue:[15] ifFalse:[super sliderThickness]! !

!GLMScrollBar methodsFor: 'stepping and presenter' stamp: 'david_roethlisberger 3/19/2009 10:40'!
step
	| action |
	action := self scrollBarAction.
	action ifNotNil:[self perform: action].! !

!GLMScrollBar methodsFor: 'testing' stamp: 'david_roethlisberger 3/19/2009 10:40'!
stepTime
	^ currentScrollDelay ifNil: [300]! !

!GLMScrollBar methodsFor: 'access' stamp: 'david_roethlisberger 3/19/2009 10:40'!
thumbColor
	"Problem: Part of the ScrollBar/Slider code uses 'slider' to mean the entire scrollbar/slider widget, and part of it uses 'slider' to mean only the draggable 'thumb'.  This should be cleaned up so that 'thumb' is used instead of 'slider' where appropriate.  For now, the meaning of thumbColor is clear, at least."

	^self sliderColor alphaMixed: 0.7 with: (Color gray: 0.95).! !

!GLMScrollBar methodsFor: 'geometry' stamp: 'david_roethlisberger 3/19/2009 10:40'!
totalSliderArea
	| upperBoundsButton |
	upperBoundsButton := upButton.
	upButton bottom > upperBoundsButton bottom
		ifTrue: [upperBoundsButton := upButton].
	^ bounds isWide
		ifTrue: [upperBoundsButton bounds topRight corner: downButton bounds bottomLeft]
		ifFalse: [upperBoundsButton bounds bottomLeft corner: downButton bounds topRight].
! !

!GLMScrollBar methodsFor: 'initialize' stamp: 'david_roethlisberger 3/19/2009 10:40'!
upArrow8Bit

	"convert to 8-bit and convert white to transparent to avoid gratuitous conversion every time we put one in an ImageMorph"

	^UpArrow8Bit ifNil: [
		UpArrow8Bit := (ColorForm mappingWhiteToTransparentFrom: UpArrow) asFormOfDepth: 8
	]! !

!GLMScrollBar methodsFor: 'scroll timing' stamp: 'david_roethlisberger 3/19/2009 10:40'!
waitForDelay1: delay1 delay2: delay2 
	"Return true if an appropriate delay has passed since the last scroll operation.
	The delay decreases exponentially from delay1 to delay2."

	| now scrollDelay |
	timeOfLastScroll isNil ifTrue: [self resetTimer].	"Only needed for old instances"
	now := Time millisecondClockValue.
	(scrollDelay := currentScrollDelay) isNil 
		ifTrue: [scrollDelay := delay1	"initial delay"].
	currentScrollDelay := scrollDelay * 9 // 10 max: delay2.	"decrease the delay"
	timeOfLastScroll := now.
	^true! !

!GLMScrollBar methodsFor: 'testing' stamp: 'david_roethlisberger 3/19/2009 10:40'!
wantsSteps
	^self scrollBarAction notNil! !

!GLMBrowser methodsFor: '*Glamour-Morphic' stamp: 'tg 5/6/2009 13:16'!
magritte
	^ self custom: GLMMagrittePresentation new! !

Announcer subclass: #GLMMorphicModel
	instanceVariableNames: 'keystrokeActions menuActions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

GLMMorphicModel subclass: #GLMListModel
	instanceVariableNames: 'items selection onSelection displaySelector searchText listSelection searchBlock singleSelection filterBlock filteredItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

!GLMListModel methodsFor: 'accessing' stamp: 'jorge.ressia 5/26/2009 15:45'!
canonicalItems
	^items! !

!GLMListModel methodsFor: 'callbacks' stamp: 'jorge.ressia 5/26/2009 13:38'!
displayListAt: anIndex
	^displaySelector glamourValue: (self items at: anIndex)! !

!GLMListModel methodsFor: 'accessing' stamp: 'david_roethlisberger 3/2/2009 11:03'!
displaySelector: aSymbol
	displaySelector := aSymbol! !

!GLMListModel methodsFor: 'associating' stamp: ''!
doSearch
	searchText isEmptyOrNil
		ifTrue: [ self executeReset ]
		ifFalse:[ 
			searchBlock isNil 
				ifTrue: [ self executeFilter ]
				ifFalse: [ self executeSearch ] ].
		
	! !

!GLMListModel methodsFor: 'associating' stamp: ''!
executeFilter
	filteredItems := filterBlock value: self searchText asString value: self canonicalItems.
	self changed: #items; changed: #selection
	! !

!GLMListModel methodsFor: 'associating' stamp: ''!
executeReset
	filteredItems := self canonicalItems.
	self changed: #items; changed: #selection
	! !

!GLMListModel methodsFor: 'associating' stamp: ''!
executeSearch
	| searchResult |
	searchResult := searchBlock value: self searchText asString value: self canonicalItems.
	listSelection := searchResult asOrderedCollection.
	onSelection ifNotNil: [onSelection value: (OrderedCollection withAll: listSelection) ].
	self changed: #items; changed: #selection
	! !

!GLMListModel methodsFor: 'accessing' stamp: 'jorge.ressia 5/26/2009 11:55'!
filterBlock: aBlock

	filterBlock := aBlock! !

!GLMListModel methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	selection := 0.
	listSelection := OrderedCollection new.
	searchText := ''
	"searchBlock := [:text :list | OrderedCollection new]"! !

!GLMListModel methodsFor: 'accessing' stamp: 'jorge.ressia 5/26/2009 15:45'!
items
	^filteredItems! !

!GLMListModel methodsFor: 'accessing' stamp: 'jorge.ressia 5/26/2009 15:46'!
items: aCollection
	items := aCollection.
	filteredItems := aCollection.! !

!GLMListModel methodsFor: 'callbacks' stamp: 'david_roethlisberger 3/2/2009 11:03'!
listAt: anIndex
	^(self items at: anIndex) "perform: displaySelector"! !

!GLMListModel methodsFor: 'callbacks' stamp: 'jorge.ressia 5/22/2009 22:32'!
listSelectionAt: anIndex
	^listSelection includes: anIndex
	"^listSelection includes: (items at: anIndex ifAbsent: [ ^ false ])."! !

!GLMListModel methodsFor: 'callbacks' stamp: 'jorge.ressia 5/26/2009 15:03'!
listSelectionAt: anIndex put: aValue
	aValue = true
		ifTrue: [listSelection add: anIndex]
		ifFalse: [listSelection
				remove: anIndex
				ifAbsent: []].
	
	onSelection ifNotNil: [onSelection value: (listSelection collect: [:eachIndex | self listAt: eachIndex])].
	self changed: #selection.
	! !

!GLMListModel methodsFor: 'callbacks' stamp: 'david_roethlisberger 2/27/2009 10:54'!
listSize
	^self items size! !

!GLMListModel methodsFor: 'updating' stamp: 'jorge.ressia 5/25/2009 13:19'!
onSelection: aBlock
	onSelection := aBlock! !

!GLMListModel methodsFor: 'accessing' stamp: 'jorge.ressia 5/26/2009 12:19'!
searchBlock: aBlock
	searchBlock := aBlock! !

!GLMListModel methodsFor: 'accessing' stamp: 'tg 5/20/2009 14:43'!
searchEnabled
	^ true! !

!GLMListModel methodsFor: 'accessing' stamp: ''!
searchText
	^ searchText! !

!GLMListModel methodsFor: 'accessing' stamp: ''!
searchText: aString
	searchText := aString.
	self doSearch
! !

!GLMListModel methodsFor: 'callbacks' stamp: 'jorge.ressia 5/26/2009 14:55'!
selection 
	^selection! !

!GLMListModel methodsFor: 'callbacks' stamp: ''!
selection: anIndex.
	selection := anIndex. 
	self changed: #selection.! !

!GLMListModel methodsFor: 'callbacks' stamp: 'jorge.ressia 5/25/2009 13:40'!
singleSelection 
	^selection! !

!GLMListModel methodsFor: 'callbacks' stamp: 'jorge.ressia 5/22/2009 23:03'!
singleSelection: anIndex.
	(anIndex == 0) ifFalse: [
		selection := anIndex.
		onSelection ifNotNil: [onSelection value: (self listAt: anIndex) ] ].
	self changed: #singleSelection.! !

!GLMMorphicModel methodsFor: 'accessing' stamp: 'david_roethlisberger 3/3/2009 16:26'!
addKeystrokeAction: anAction
	keystrokeActions add: anAction! !

!GLMMorphicModel methodsFor: 'accessing' stamp: 'tg 5/22/2009 23:03'!
addMenuAction: anAction
	menuActions add: anAction! !

!GLMMorphicModel methodsFor: 'callbacks' stamp: 'tg 5/22/2009 23:03'!
executeMenuAction: anAction
	self announce: (GLMMenuItemSelected action: anAction)! !

!GLMMorphicModel methodsFor: 'initialization' stamp: 'tg 5/22/2009 23:04'!
initialize
	super initialize.
	keystrokeActions := OrderedCollection new.
	menuActions := OrderedCollection new.! !

!GLMMorphicModel methodsFor: 'callbacks' stamp: 'david_roethlisberger 3/3/2009 16:24'!
keystroke: aChar from: aMorph
	| action |
	action := keystrokeActions detect: [:a | a shortcut = aChar] ifNone: [nil].
	action ifNotNil: [self announce: (GLMKeyStroke action: action)].! !

!GLMMorphicModel methodsFor: 'callbacks' stamp: 'tg 5/22/2009 23:03'!
menu: aMenuMorph
	menuActions do: [:action |
		aMenuMorph add: action title target: self selector: #executeMenuAction: argument: action].
	^aMenuMorph! !

GLMMorphicModel subclass: #GLMTextModel
	instanceVariableNames: 'text onTextChanged'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

!GLMTextModel methodsFor: 'callbacks' stamp: 'tg 5/12/2009 01:10'!
accept: aText notifying: anObject 
	aText asString withBlanksTrimmed ifEmpty: [^self].
	onTextChanged ifNotNil: [onTextChanged value: aText].
	text := aText.
	self changed: #text! !

!GLMTextModel methodsFor: 'callbacks' stamp: 'david_roethlisberger 3/2/2009 16:22'!
keystroke: anEvent from: aMorph
	anEvent anyModifierKeyPressed ifTrue: [super keystroke: anEvent keyCharacter from: aMorph]! !

!GLMTextModel methodsFor: 'accessing' stamp: 'david_roethlisberger 2/27/2009 12:08'!
onTextChanged: aBlock
	onTextChanged := aBlock! !

!GLMTextModel methodsFor: 'callbacks' stamp: 'tg 5/14/2009 22:10'!
selectSilently: anIndex
	self changed: #selection! !

!GLMTextModel methodsFor: 'callbacks' stamp: 'david_roethlisberger 2/27/2009 11:50'!
selection
	^ 1 to: 0! !

!GLMTextModel methodsFor: 'callbacks' stamp: 'david_roethlisberger 2/27/2009 11:50'!
text
	^text! !

!GLMTextModel methodsFor: 'accessing' stamp: 'david_roethlisberger 2/27/2009 11:51'!
text: aTextOrString
	text := aTextOrString! !

GLMMorphicModel subclass: #GLMTreeModel
	instanceVariableNames: 'roots selection childrenBlock onSelection displaySelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

!GLMTreeModel methodsFor: 'accessing' stamp: 'david_roethlisberger 2/27/2009 14:24'!
childrenBlock
	^childrenBlock! !

!GLMTreeModel methodsFor: 'accessing' stamp: 'david_roethlisberger 2/27/2009 14:24'!
childrenBlock: aBlock
	childrenBlock := aBlock! !

!GLMTreeModel methodsFor: 'accessing' stamp: 'tg 5/6/2009 10:56'!
displaySelector: aSymbol
	displaySelector := aSymbol! !

!GLMTreeModel methodsFor: 'callbacks' stamp: 'david_roethlisberger 2/27/2009 13:56'!
geSelectionPath
	^self selection! !

!GLMTreeModel methodsFor: 'callbacks' stamp: 'david_roethlisberger 2/27/2009 14:32'!
getChildrenOf: aNode
	^childrenBlock value: aNode! !

!GLMTreeModel methodsFor: 'callbacks' stamp: 'tg 5/6/2009 10:56'!
getLabelOf: anObject
	^ displaySelector glamourValue: anObject! !

!GLMTreeModel methodsFor: 'callbacks' stamp: 'david_roethlisberger 2/27/2009 14:32'!
hasChildren: aNode
	^(self getChildrenOf: aNode) notEmpty! !

!GLMTreeModel methodsFor: 'private' stamp: 'david_roethlisberger 2/27/2009 14:12'!
nodeAt: pathCollection
	^self nodeAt: pathCollection allButFirst for: self roots! !

!GLMTreeModel methodsFor: 'private' stamp: 'david_roethlisberger 2/27/2009 14:26'!
nodeAt: pathCollection for: childrenCollection
	| node |
	node := childrenCollection at: pathCollection first.
	^pathCollection size > 1 ifTrue: [self nodeAt: pathCollection allButFirst for: (childrenBlock value: node)] ifFalse: [node]! !

!GLMTreeModel methodsFor: 'accessing' stamp: 'david_roethlisberger 2/27/2009 14:38'!
onSelection: aBlock
	onSelection := aBlock! !

!GLMTreeModel methodsFor: 'callbacks' stamp: 'david_roethlisberger 2/27/2009 13:54'!
roots
	^roots! !

!GLMTreeModel methodsFor: 'accessing' stamp: 'david_roethlisberger 2/27/2009 14:22'!
roots: aCollectionOrElement
	roots := aCollectionOrElement isCollection ifTrue: [aCollectionOrElement] ifFalse: [OrderedCollection with: aCollectionOrElement]! !

!GLMTreeModel methodsFor: 'accessing' stamp: 'david_roethlisberger 2/27/2009 13:57'!
selection
	^selection! !

!GLMTreeModel methodsFor: 'callbacks' stamp: 'david_roethlisberger 2/27/2009 14:39'!
selection: aNode
     selection := aNode.
	onSelection ifNotNil: [onSelection value: aNode].
	self changed: #selection.! !

Object subclass: #GLMMorphicExamples
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Morphic'!

!GLMMorphicExamples methodsFor: 'examples' stamp: 'tg 5/12/2009 12:31'!
demoBrowser
	"
		self new demoBrowser openOn: Smalltalk organization categories
	"
	| browser |
	browser := GLMTableLayoutBrowser new.
	^ browser ! !

!GLMMorphicExamples methodsFor: 'examples' stamp: 'tg 5/12/2009 12:30'!
demoNavigator
	"
		self new demoNavigator openOn: Smalltalk organization categories
	"
	| browser |
	browser := GLMTableLayoutBrowser new.
	^ browser ! !

!GLMMorphicExamples methodsFor: 'examples' stamp: 'tg 5/12/2009 01:08'!
mooseFinder
	"
		self new mooseFinder openOn: MooseModel root
	"

	| browser pragmas |
	browser := GLMFinder new.
	
	browser list
		display: [:each | each entities];
		title: 'Group';
		whenKindOf: MooseGroup.
	browser list
		title: 'Entity';
		display:  [ :anObject | 
			pragmas := anObject class withAllSuperclasses flatCollect: [:each |
				Pragma allNamed: #navigation: in: each].
			(pragmas collect: [:each | anObject perform: each selector ])
				select: [ :each | each isCollection  and: [ each isEmpty not ] ].
		].
	browser mondrian
		painting: [ :view :each | each viewBlueprintOn: view ];
		title: 'Blueprint';
		whenKindOf: FAMIXClass.
	browser text
		display: #sourceText;
		title: 'Method Source';
		whenKindOf: FAMIXMethod.
	browser text
		display: #sourceText;
		title: 'Class Source';
		whenKindOf: FAMIXClass.
	browser text
		display: [:entity | 'self' ];
		title: 'Evaluator';
		update: #selection on: $o with: [ :text :entity | Compiler evaluate: text selectedText for: entity logged: false ];
		act: [ :text :entity | (Compiler evaluate: text selectedText for: entity logged: false) inspect ] on: $/.
	
	^ browser! !

!GLMMorphicExamples methodsFor: 'examples' stamp: 'tg 5/11/2009 17:53'!
moosePreviewFinder
	"
		self new moosePreviewFinder openOn: MooseModel root
	"

	| browser |
	browser := GLMTableLayoutBrowser new.
	browser row: #visual; row: #finder; row: #textual.
	browser showOn: #finder; using: [
		browser custom: self mooseFinder.
	].
	browser showOn: #visual; from: #finder; using: [
		browser mondrian
			painting: [ :view :classes | classes viewSystemComplexityOn: view ];
			whenKindOf: FAMIXClassGroup.
	].
	browser showOn: #textual; from: #finder; using: [
		browser text
			display: [:element | element mooseName]
	].

	^ browser! !

!GLMMorphicExamples methodsFor: 'examples' stamp: 'tg 5/6/2009 15:08'!
pierFinder
	"
		self new pierFinder openOn: PRKernel instances anyOne root.
	"
	| browser |
	browser := GLMFinder new.
	browser list
		display: [:each | each enumerator contents];
		format: [:each | each title];
		title: 'Navigation'.
	browser magritte title: 'Editor'.
	browser mondrian
			title: 'Tree';
			painting: [:view :each | 
				view nodeShape: (MORectangleShape withBorder height: [:x | x plainText size // 10]).
				view nodes: each enumerator everything contents.
				view edgesFrom: #parent.
				view treeLayout].
	^ browser ! !

!GLMMorphicExamples methodsFor: 'examples' stamp: 'tg 5/6/2009 13:31'!
pierTreeEditor

	"
		self new pierTreeEditor openOn: PRKernel instances anyOne root.
	"

	| browser |
	browser := GLMTableLayoutBrowser new.
	browser column: #navigation; column: #editor span: 4.
	browser showOn: #navigation; using: [
		browser tree
			children: [:each | each enumerator contents];
			format: [:each | each title ];
			title: 'Navigation'.
	].
	browser showOn: #editor; from: #navigation; using: [
		browser magritte
			title: 'Editor'.
	].
	^ browser ! !

!GLMMorphicExamples methodsFor: 'examples' stamp: 'david_roethlisberger 3/19/2009 10:35'!
simpleFinder

	"
		self new simpleFinder openOn: #(1 2 3 4 5).
	"

	| finder |
	finder := GLMFinder new.
	finder list
			when: [:each | each isCollection].
	finder list
			title: 'List';
			display: [:each | each to: each];
			when: [:each | each isCollection not].
	finder list
			title: 'List * 10';
			display: [:each | each to: each*10];
			when: [:each | each isCollection not].
	finder text
			title: 'Text'.	
	^ finder ! !

!GLMMorphicExamples methodsFor: 'examples' stamp: 'tg 5/13/2009 16:40'!
stCodeBrowser	
	"
		| browser |
		browser := self new stCodeBrowser.
		browser openOn: Smalltalk organization categories.
	"

	| browser |

	browser := GLMTableLayoutBrowser new.
	browser row: #navigation; row: #sourceCode.

	browser showOn: #navigation; using: [
		browser custom: self stCodeNavigator.
	].

	browser
		showOn: #sourceCode;
		from: #navigation->#selectedClass;
		from: #navigation->#selectedMethod;
		from: #navigation->#selectedClassCategory;
		using: [
			browser text
				title: 'Method source';
				act: [:pane | Transcript show: 'action d'; cr. ] on: $d entitled: 'some action'; 
				display: [ :class :method | class sourceCodeAt: method ];
				when: [ :class :method | class notNil & method notNil ].
			browser text
				title: 'Class definition';
				display: [ :class | class definition ];
				when: [:class | class notNil].
			browser text
				title: 'Class comment';
				display: [ :class | class comment];
				when: [:class | class notNil].
			browser mondrian
				title: 'System Complexity';
				painting: [ :view :class :method :cat | 
					view nodes: ((Smalltalk organization listAtCategoryNamed: cat) collect: [:className | Smalltalk classNamed: className]).
					view edgesFrom: [:x | x superclass].
					view treeLayout. ];
				when: [:class :method :cat | cat notNil].
		].

	^browser! !

!GLMMorphicExamples methodsFor: 'examples' stamp: 'tg 6/5/2009 02:01'!
stCodeNavigator	
	"
		| browser |
		browser := self new stCodeNavigator.
		browser openOn: Smalltalk organization categories.
		((browser paneNamed: #classCats) port: #selection) value: Smalltalk organization categories first
	"

	| browser |
	browser := GLMTableLayoutBrowser new.
	browser column: #classCats; column:#classes; column:#categories; column: #methods.

	browser showOn: #classCats; using: [
		browser list
			filterOn: [:text :list | list select: [:each | text match: each ]].
	].

	browser showOn: #classes; from: #classCats; using: [
		browser list
			display: [ :classCat |  (Smalltalk organization listAtCategoryNamed: classCat) collect: [:className | Smalltalk classNamed: className] ];
			when: [ :classCat | classCat notNil ].
	].

	browser showOn: #categories; from: #classes; using: [
		browser list display: [ :class | class organization categories ].
	].

	browser 
		showOn: #methods; from: #classes; from: #categories;
		using: [
			browser list
				display: [  :class :category | (class organization listAtCategoryNamed: category) asSortedCollection ];
				when: [  :class :category | class notNil and: [ category notNil ] ].
			browser list
				when: [ :class :category | class notNil and: [ category isNil ] ];
				display: [ :class :category | class selectors asSortedCollection ].
	].

	browser sendToOutside: #selectedClassCategory from: #classCats.
	browser sendToOutside: #selectedClass from: #classes.
	browser sendToOutside: #selectedMethod from: #methods.

	^browser! !

!GLMMorphicExamples methodsFor: 'examples' stamp: 'tg 5/11/2009 23:10'!
toyBrowser
	"
		self new toyBrowser openOn: Smalltalk organization categories
	"

	| browser |
	browser := GLMTableLayoutBrowser new.
	browser column: #categories; column: #classes; column: #methods.
	browser showOn: #classes; from: #categories; using: [
		browser list
			display: [:category | (Smalltalk organization listAtCategoryNamed: category) collect: [:className | Smalltalk classNamed: className] ]
	].
	browser showOn: #methods; from: #classes; using: [
		browser tree
			children: [ :class  | class selectors asSortedCollection ].
	].
	^ browser! !

!GLMMorphicExamples methodsFor: 'examples' stamp: 'tg 5/11/2009 22:25'!
toyNavigatorBrowser
	"
		| browser |
		browser := self new toyNavigatorBrowser.
		browser openOn: Smalltalk organization categories.
		((browser paneNamed: #categories) port: #selection) value: Smalltalk organization categories first.
	"

	| browser classes |
	browser := GLMTableLayoutBrowser new.
	browser column: #categories; column: #classes; column: #metCategories; column: #methods.
	browser showOn: #classes->#entity; from: #categories->#selection; using: [
		browser list
			title: 'Classes';
			display: [:category | (Smalltalk organization listAtCategoryNamed: category) collect: [:className | Smalltalk classNamed: className] ].
		browser mondrian
			title: 'System Complexity';
			painting: [:view :category | 
				classes := (Smalltalk organization listAtCategoryNamed: category) collect: [:className | Smalltalk classNamed: className].
				view nodes: classes.
				view edgesFrom: #superclass.
				view treeLayout
			]
	].
	browser showOn: #metCategories; from: #classes; using: [
		browser list
			display: [:class | class organization categories ]
	].
	browser showOn: #methods; from: #classes; from: #metCategories; using: [
		browser list
			display: [  :class :category | (class organization listAtCategoryNamed: category) asSortedCollection ];
			when: [  :class :category | class notNil and: [ category notNil ] ].
		browser list
			display: [ :class :category | class selectors asSortedCollection ];
			when: [ :class :category | class notNil and: [ category isNil ] ].
	].
	^ browser! !
GLMScrollBar initialize!
